@@ -13,16 +13,17 @@
 ##############################################################################
 """Extensions for z3c.etestbrowser
 
-$Id: testing.py 71409 2006-12-05 07:44:33Z ctheune $
+$Id: testing.py 79569 2007-09-11 13:56:09Z ctheune $
 """
 
+import re
 import StringIO
 import lxml.etree
 
 import zope.testbrowser.testing
 
 
-html_parser = lxml.etree.HTMLParser()
+RE_CHARSET = re.compile('.*;charset=(.*)')
 
 
 class ExtendedTestBrowser(zope.testbrowser.testing.Browser):
@@ -34,6 +35,8 @@ class ExtendedTestBrowser(zope.testbrowser.testing.Browser):
 
     """
 
+    xml_strict = False
+
     _etree = None
 
     @property
@@ -42,8 +45,21 @@ class ExtendedTestBrowser(zope.testbrowser.testing.Browser):
             return self._etree
         # I'm not using any internal knowledge about testbrowser
         # here, to avoid breakage. Memory usage won't be a problem.
-        content = StringIO.StringIO(self.contents)
-        self._etree = lxml.etree.parse(content, html_parser)
+        if self.xml_strict:
+            self._etree = lxml.etree.XML(self.contents)
+        else:
+            # This is a workaround against the broken fallback for 
+            # encoding detection of libxml2.
+            # We have a chance of knowing the encoding as Zope states this in
+            # the content-type response header.
+            content = self.contents
+            content_type = self.headers['content-type']
+            match = RE_CHARSET.match(content_type)
+            if match is not None:
+                charset = match.groups()[0]
+                content = content.decode(charset)
+            self._etree = lxml.etree.HTML(content)
+
         return self._etree
 
     def _changed(self):
